@@ -1,15 +1,24 @@
-import base64
-import hashlib
+"""Unofficial wrapper for iFunny's private API
+
+Example:
+
+from ifunnyapi.api import IFAPI
+api = IFAPI("token")
+for post in api.user_posts(user_id=api.account["id"]):
+    api.smile_post(post_id=post["id"])
+for feat in api.featured(limit=1):
+    api.comment("nice feature!", post_id=feat["id"])
+"""
+
 import io
 import json
-import requests
-import uuid
-from PIL import Image, UnidentifiedImageError
-from time import sleep
 from typing import Generator, List, Union
+
+from PIL import Image, UnidentifiedImageError
+import requests
+
 from .auth import AuthBearer
 from .enums import IFChannel, IFPostVisibility, IFReportType
-from .exceptions import APIError
 from .utils import api_request
 
 
@@ -34,8 +43,8 @@ class _IFBaseAPI:
             JSON dictionary of request output.
         """
 
-        r = requests.get(IFAPI.BASE + path, auth=self.auth, **kwargs)
-        return r.json()
+        req = requests.get(IFAPI.BASE + path, auth=self.auth, **kwargs)
+        return req.json()
 
     @api_request
     def _post(self, path: str, **kwargs) -> dict:
@@ -49,8 +58,8 @@ class _IFBaseAPI:
             JSON dictionary of request output.
         """
 
-        r = requests.post(IFAPI.BASE + path, auth=self.auth, **kwargs)
-        return r.json()
+        req = requests.post(IFAPI.BASE + path, auth=self.auth, **kwargs)
+        return req.json()
 
     @api_request
     def _put(self, path: str, **kwargs) -> dict:
@@ -64,8 +73,8 @@ class _IFBaseAPI:
             JSON dictionary of request output.
         """
 
-        r = requests.put(IFAPI.BASE + path, auth=self.auth, **kwargs)
-        return r.json()
+        req = requests.put(IFAPI.BASE + path, auth=self.auth, **kwargs)
+        return req.json()
 
     @api_request
     def _delete(self, path: str, **kwargs) -> dict:
@@ -79,8 +88,8 @@ class _IFBaseAPI:
             JSON dictionary of request output.
         """
 
-        r = requests.delete(IFAPI.BASE + path, auth=self.auth, **kwargs)
-        return r.json()
+        req = requests.delete(IFAPI.BASE + path, auth=self.auth, **kwargs)
+        return req.json()
 
     @property
     def account(self, **kwargs) -> dict:
@@ -122,7 +131,7 @@ class _IFBaseAPI:
         return self._get(f"/content/{post_id}", **kwargs)["data"]
 
     def comment_info(self, *, post_id: str, comment_id: str, **kwargs) -> dict:
-        """Retrieve iFunny post.
+        """Retrieve iFunny comment.
 
         Args:
             post_id: iFunny ID of post with comment to retrieve.
@@ -137,6 +146,18 @@ class _IFBaseAPI:
             f"/content/{post_id}/comments/{comment_id}", **kwargs
         )["data"]
 
+    def channels_info(self, **kwargs) -> List[dict]:
+        """Retrieve iFunny channels.
+
+        Args:
+            **kwargs: Arbitrary keyword arguments passed to requests.
+
+        Returns:
+            List of JSON dictionary of iFunny channels.
+        """
+
+        return self._get("/channels", **kwargs)["data"]["channels"]["items"]
+
     def _get_paging_items(
             self,
             path: str,
@@ -157,17 +178,18 @@ class _IFBaseAPI:
             List of JSON dictionaries of paging items.
         """
 
-        def get_next(r: dict) -> int:
-            return r["data"][key]["paging"]["cursors"]["next"]
+        def get_next(jso: dict) -> int:
+            return jso["data"][key]["paging"]["cursors"]["next"]
 
-        def has_next(r: dict) -> bool:
-            return r["data"][key]["paging"]["hasNext"]
+        def has_next(jso: dict) -> bool:
+            return jso["data"][key]["paging"]["hasNext"]
 
-        def get_items(r: dict) -> list:
-            return r["data"][key]["items"]
+        def get_items(jso: dict) -> list:
+            return jso["data"][key]["items"]
 
         lnone = limit is None
         ilim = 100 if lnone or limit > 100 else limit
+        val = rem = 0
         batch = self._get(path, params={"limit": ilim}, **kwargs)
         items = get_items(batch)
 
@@ -484,9 +506,8 @@ class _IFBaseAPI:
 
         iterator = iter(int, 1) if limit is None else range(limit)
         for _ in iterator:
-            r = self._get("/feeds/featured", params={"limit": 1}, **kwargs)
-            feat = r["data"]["content"]["items"]
-            feat = feat[0]
+            jso = self._get("/feeds/featured", params={"limit": 1}, **kwargs)
+            feat = jso["data"]["content"]["items"][0]
             if read:
                 self._put(
                     f"/reads/{feat['id']}",
@@ -512,9 +533,12 @@ class _IFBaseAPI:
 
         iterator = iter(int, 1) if limit is None else range(limit)
         for _ in iterator:
-            r = self._post("/feeds/collective", params={"limit": 1}, **kwargs)
-            coll = r["data"]["content"]["items"]
-            coll = coll[0]
+            jso = self._post(
+                "/feeds/collective",
+                params={"limit": 1},
+                **kwargs
+            )
+            coll = jso["data"]["content"]["items"][0]
             yield coll
 
     def subscriptions(
@@ -537,9 +561,8 @@ class _IFBaseAPI:
 
         iterator = iter(int, 1) if limit is None else range(limit)
         for _ in iterator:
-            r = self._get("/timelines/home", params={"limit": 1}, **kwargs)
-            subscr = r["data"]["content"]["items"]
-            subscr = subscr[0]
+            jso = self._get("/timelines/home", params={"limit": 1}, **kwargs)
+            subscr = jso["data"]["content"]["items"][0]
             if read:
                 self._put(
                     f"/reads/{subscr['id']}",
@@ -565,11 +588,35 @@ class _IFBaseAPI:
 
         iterator = iter(int, 1) if limit is None else range(limit)
         for _ in iterator:
-            r = self._get("/feeds/popular", params={"limit": 1}, **kwargs)
-            pop = r["data"]["content"]["items"]
-            pop = pop[0]
+            jso = self._get("/feeds/popular", params={"limit": 1}, **kwargs)
+            pop = jso["data"]["content"]["items"][0]
             yield pop
 
+    def digest_posts(
+            self,
+            *,
+            day: int,
+            month: int,
+            year: int,
+            **kwargs
+    ) -> List[dict]:
+        """Retrieve iFunny posts from specified digest.
+
+        Args:
+            day: Digest day.
+            month: Digest month.
+            year: Digest year.
+            **kwargs: Arbitrary keyword arguments passed to requests.
+
+        Returns:
+           List of JSON dictionaries of iFunny posts from specified digest.
+        """
+
+        return self._get(
+            f"/digests/weekly-iFunny-{year}{month:02d}{day:02d}",
+            **kwargs
+        )["data"]["items"]
+
     def upload(
             self,
             media: Union[bytes, str],
@@ -588,14 +635,16 @@ class _IFBaseAPI:
             **kwargs: Arbitrary keyword arguments passed to requests.
         """
 
-        media = media if type(media) is bytes else open(media, "rb").read()
+        if isinstance(media, str):
+            with open(media, "rb") as file:
+                media = file.read()
         try:
-            im = Image.open(io.BytesIO(media))
+            image = Image.open(io.BytesIO(media))
         except UnidentifiedImageError:
             mtype = "video_clip"
             ftype = "video"
         else:
-            mtype = "gif" if im.format == "GIF" else "pic"
+            mtype = "gif" if image.format == "GIF" else "pic"
             ftype = "image"
         self._post(
             "/content",
@@ -753,6 +802,46 @@ class _IFBaseAPI:
             **kwargs
         )
 
+    def pin_post(self, *, post_id: str, **kwargs):
+        """Pin a post.
+
+        Args:
+            post_id: iFunny ID of post to pin.
+            **kwargs: Arbitrary keyword arguments passed to requests.
+        """
+
+        self._post(f"/content/{post_id}/pinned", **kwargs)
+
+    def unpin_post(self, *, post_id: str, **kwargs):
+        """Unpin a post.
+
+        Args:
+            post_id: iFunny ID of post to unpin.
+            **kwargs: Arbitrary keyword arguments passed to requests.
+        """
+
+        self._delete(f"/content/{post_id}/pinned", **kwargs)
+
+    def republish_post(self, *, post_id: str, **kwargs):
+        """Republish a post.
+
+        Args:
+            post_id: iFunny ID of post to republish.
+            **kwargs: Arbitrary keyword arguments passed to requests.
+        """
+
+        self._post(f"/content/{post_id}/republished", **kwargs)
+
+    def unrepublish_post(self, *, post_id: str, **kwargs):
+        """Unrepublish a post.
+
+        Args:
+            post_id: iFunny ID of post to unrepublish.
+            **kwargs: Arbitrary keyword arguments passed to requests.
+        """
+
+        self._delete(f"/content/{post_id}/republished", **kwargs)
+
     def smile_post(self, *, post_id: str, **kwargs):
         """Smile a post.
 
@@ -924,108 +1013,16 @@ class _IFBaseAPI:
 class IFAPI(_IFBaseAPI):
     """Public API class, includes extra features"""
 
-    CLIENTID = "JuiUH&3822"
-    CLIENTSEC = "HuUIC(ZQ918lkl*7"
-
-    @classmethod
-    def generate_basic_token(cls, register: bool = False) -> str:
-        """Generate an iFunny basic token.
-
-        Args:
-            register: Option to send a request to register the token.
-
-        Returns:
-            iFunny basic token.
-        """
-
-        hexstr = str(uuid.uuid4()).encode("utf-8").hex().upper()
-        hid = f"{hexstr}_{cls.CLIENTSEC}"
-        hashdec = f"{hexstr}:{cls.CLIENTID}:{cls.CLIENTSEC}"
-        hashenc = hashlib.sha1(hashdec.encode("utf-8")).hexdigest()
-        btoken = base64.b64encode(bytes(f"{hid}:{hashenc}", "utf-8")).decode()
-
-        if register:
-            requests.get(
-                "http://geoip.ifunny.co/",
-                cookies={"device_id": hexstr},
-                headers={"Cookie": f"device_id={hexstr}", "User-Agent": "*"},
-            )
-
-        return btoken
-
-    @classmethod
-    def _req_auth(cls, btoken: str, email: str, password: str) -> dict:
-        """Request an iFunny bearer authorization token with credentials.
-
-        Args:
-            btoken: iFunny basic token.
-            email: iFunny account email.
-            password: iFunny account password.
-
-        Returns:
-            JSON dictionary containing iFunny bearer authorization token.
-        """
-
-        r = requests.post(
-            cls.BASE + "/oauth2/token",
-            headers={"Authorization": "Basic " + btoken},
-            data={"grant_type": "password",
-                  "username": email,
-                  "password": password}
-        )
-        return r.json()
-
-    @classmethod
-    def from_creds(cls, email: str, password: str, primer: int = 10):
-        """Instantiate iFunnyAPI from credentials rather than token.
-
-        Args:
-            email: iFunny account email.
-            password: iFunny account password.
-            primer: Time to sleep after primer request.
-
-        Returns:
-            iFunnyAPI instance from specified credentials.
-        """
-
-        btoken = cls.generate_basic_token()
-        cls._req_auth(btoken, email, password)  # Prime the token
-        sleep(primer)
-        r = cls._req_auth(btoken, email, password)
-        if "error" in r:
-            raise APIError(r["status"], r["error_description"])
-        token = r["access_token"]
-        return cls(token)
-
-    @classmethod
-    def generate_account(
-            cls,
-            email: str,
-            nick: str,
-            password: str,
-            accepted_mailing: bool = False
-    ):
-        """This method is not yet implemented.
-
-        Args:
-            email:
-            nick:
-            password:
-            accepted_mailing:
-        """
-
-        raise NotImplementedError("generate_account is not yet implemented")
-
     @staticmethod
-    def crop_ifunny_watermark(im: Image) -> Image:
+    def crop_ifunny_watermark(image: Image) -> Image:
         """Crop the iFunny watermark from an image.
 
         Args:
-            im: PIL.Image instance with iFunny watermark.
+            image: PIL.Image instance with iFunny watermark.
 
         Returns:
             Image with bottom 20 pixels (watermark) cropped.
         """
 
-        width, height = im.size
-        return im.crop((0, 0, width, height - 20))
+        width, height = image.size
+        return image.crop((0, 0, width, height - 20))
