@@ -1,8 +1,11 @@
+"""Raw-value enums used by iFunny."""
+
 from enum import Enum
 
 
 class IFChannel(Enum):
-    COMMUNITY_COMMERCIALS = "5fd3ea0dc6ff1f1a8008c4cc"
+    """iFunny channel names to channel IDs."""
+
     CYBERPUNK = "5fd9a9533ea93f29fe1af892"
     WTF = "5e28bbc2a264f60031232af1"
     ANIMALS = "5a05c3e640fe2732008b456d"
@@ -16,14 +19,18 @@ class IFChannel(Enum):
 
 
 class IFPostVisibility(Enum):
+    """iFunny post visibility option string representations."""
+
     PUBLIC = "public"
     SUBSCRIBERS_ONLY = "subscribers"
 
 
 class IFReportType(Enum):
+    """iFunny report type string representations."""
+
     HATE_SPEECH = "hate"
     NUDITY = "nude"
     SPAM = "spam"
     TARGETED_ABUSE = "target"
     THREATS_OF_HARM = "harm"
-    BANNER_ISSUES = "banner"  # This may be incorrect
+    BANNER_ISSUES = "banner"
