@@ -1,12 +1,15 @@
+"""Custom exceptions used by ifunnyapi."""
+
+
 class IFAPIException(Exception):
     """Base ifunnyapi exception class."""
 
 
 class APIError(IFAPIException):
-    """Raised when an API request retrieves an error"""
+    """Raised when an API request retrieves an error."""
 
     def __init__(self, status: int, desc: str):
-        super(APIError, self).__init__()
+        super().__init__()
         self.status = status
         self.desc = desc
 
